@@ -2,14 +2,11 @@ import numpy as np
 
 import collections
 
-from .constants import log
-
-_fcl_exists = True
 try:
-    import fcl  # pip install python-fcl
+    # pip install python-fcl
+    import fcl
 except BaseException:
-    log.warning('No FCL -- collision checking will not work')
-    _fcl_exists = False
+    fcl = None
 
 
 class ContactData(object):
@@ -147,7 +144,7 @@ class CollisionManager(object):
         """
         Initialize a mesh-mesh collision manager.
         """
-        if not _fcl_exists:
+        if fcl is None:
             raise ValueError('No FCL Available!')
         # {name: {geom:, obj}}
         self._objs = {}
@@ -178,7 +175,7 @@ class CollisionManager(object):
         mesh : Trimesh object
           The geometry of the collision object
         transform : (4,4) float
-          Homogenous transform matrix for the object
+          Homogeneous transform matrix for the object
         """
 
         # if no transform passed, assume identity transform
@@ -235,7 +232,7 @@ class CollisionManager(object):
         name : str
           An identifier for the object already in the manager
         transform : (4,4) float
-          A new homogenous transform matrix for the object
+          A new homogeneous transform matrix for the object
         """
         if name in self._objs:
             o = self._objs[name]['obj']
@@ -245,8 +242,11 @@ class CollisionManager(object):
         else:
             raise ValueError('{} not in collision manager!'.format(name))
 
-    def in_collision_single(self, mesh, transform=None,
-                            return_names=False, return_data=False):
+    def in_collision_single(self,
+                            mesh,
+                            transform=None,
+                            return_names=False,
+                            return_data=False):
         """
         Check a single object for collisions against all objects in the
         manager.
@@ -256,7 +256,7 @@ class CollisionManager(object):
         mesh : Trimesh object
           The geometry of the collision object
         transform : (4,4) float
-          Homogenous transform matrix
+          Homogeneous transform matrix
         return_names : bool
           If true, a set is returned containing the names
           of all objects in collision with the object
@@ -268,10 +268,10 @@ class CollisionManager(object):
         is_collision : bool
           True if a collision occurs and False otherwise
         names : set of str
-          The set of names of objects that collided with the
+          [OPTIONAL] The set of names of objects that collided with the
           provided one
         contacts : list of ContactData
-          All contacts detected
+          [OPTIONAL] All contacts detected
         """
         if transform is None:
             transform = np.eye(4)
@@ -455,7 +455,7 @@ class CollisionManager(object):
         mesh : Trimesh object
           The geometry of the collision object
         transform : (4,4) float
-          Homogenous transform matrix for the object
+          Homogeneous transform matrix for the object
         return_names : bool
           If true, return name of the closest object
         return_data : bool
