@@ -8,11 +8,11 @@ Create meshes from primitives, or with operations.
 from .base import Trimesh
 from .constants import log, tol
 from .triangles import normals
-from .geometry import faces_to_edges
+from .geometry import faces_to_edges, align_vectors
 
 from . import util
 from . import grouping
-from . import transformations
+from . import transformations as tf
 
 import numpy as np
 import collections
@@ -21,11 +21,14 @@ try:
     # shapely is a soft dependency
     from shapely.geometry import Polygon
     from shapely.wkb import loads as load_wkb
-except BaseException:
+except BaseException as E:
     # shapely will sometimes raise OSErrors
     # on import rather than just ImportError
-    log.warning('shapely.geometry.Polygon not available!',
-                exc_info=True)
+    from . import exceptions
+    # re-raise the exception when someone tries
+    # to use the module that they don't have
+    Polygon = exceptions.closure(E)
+    load_wkb = exceptions.closure(E)
 
 
 def validate_polygon(obj):
@@ -64,6 +67,8 @@ def validate_polygon(obj):
 
 def extrude_polygon(polygon,
                     height,
+                    transform=None,
+                    triangle_args=None,
                     **kwargs):
     """
     Extrude a 2D shapely polygon into a 3D mesh
@@ -74,6 +79,8 @@ def extrude_polygon(polygon,
       2D geometry to extrude
     height : float
       Distance to extrude polygon along Z
+    triangle_args : str or None
+      Passed to triangle
     **kwargs:
         passed to Trimesh
 
@@ -82,10 +89,12 @@ def extrude_polygon(polygon,
     mesh : trimesh.Trimesh
       Resulting extrusion as watertight body
     """
-    vertices, faces = triangulate_polygon(polygon, **kwargs)
+    vertices, faces = triangulate_polygon(
+        polygon, triangle_args=triangle_args, **kwargs)
     mesh = extrude_triangulation(vertices=vertices,
                                  faces=faces,
                                  height=height,
+                                 transform=transform,
                                  **kwargs)
     return mesh
 
@@ -132,11 +141,11 @@ def sweep_polygon(polygon,
 
     # Compute 3D locations of those vertices
     verts_3d = np.c_[verts_2d, np.zeros(n)]
-    verts_3d = transformations.transform_points(verts_3d, tf_mat)
+    verts_3d = tf.transform_points(verts_3d, tf_mat)
     base_verts_3d = np.c_[base_verts_2d,
                           np.zeros(len(base_verts_2d))]
-    base_verts_3d = transformations.transform_points(base_verts_3d,
-                                                     tf_mat)
+    base_verts_3d = tf.transform_points(base_verts_3d,
+                                        tf_mat)
 
     # keep matching sequence of vertices and 0- indexed faces
     vertices = [base_verts_3d]
@@ -160,11 +169,11 @@ def sweep_polygon(polygon,
 
         # Rotate if needed
         if angles is not None:
-            tf_mat = transformations.rotation_matrix(angles[i],
-                                                     norms[i],
-                                                     path[i])
-            verts_3d_prev = transformations.transform_points(verts_3d_prev,
-                                                             tf_mat)
+            tf_mat = tf.rotation_matrix(angles[i],
+                                        norms[i],
+                                        path[i])
+            verts_3d_prev = tf.transform_points(verts_3d_prev,
+                                                tf_mat)
 
         # Project vertices onto plane in 3D
         ds = np.einsum('ij,j->i', (path[i + 1] - verts_3d_prev), norms[i])
@@ -204,6 +213,7 @@ def sweep_polygon(polygon,
 def extrude_triangulation(vertices,
                           faces,
                           height,
+                          transform=None,
                           **kwargs):
     """
     Turn a 2D triangulation into a watertight Trimesh.
@@ -237,7 +247,7 @@ def extrude_triangulation(vertices,
 
     # make sure triangulation winding is pointing up
     normal_test = normals(
-        [util.three_dimensionalize(vertices[faces[0]])[1]])[0]
+        [util.stack_3D(vertices[faces[0]])])[0]
 
     normal_dot = np.dot(normal_test,
                         [0.0, 0.0, np.sign(height)])[0]
@@ -253,7 +263,8 @@ def extrude_triangulation(vertices,
     # edges which only occur once are on the boundary of the polygon
     # since the triangulation may have subdivided the boundary of the
     # shapely polygon, we need to find it again
-    edges_unique = grouping.group_rows(edges_sorted, require_count=1)
+    edges_unique = grouping.group_rows(
+        edges_sorted, require_count=1)
 
     # (n, 2, 2) set of line segments (positions, not references)
     boundary = vertices[edges[edges_unique]]
@@ -270,7 +281,7 @@ def extrude_triangulation(vertices,
     vertical_faces = vertical_faces.reshape((-1, 3))
 
     # stack the (n,2) vertices with zeros to make them (n, 3)
-    vertices_3D = util.three_dimensionalize(vertices, return_2D=False)
+    vertices_3D = util.stack_3D(vertices)
 
     # a sequence of zero- indexed faces, which will then be appended
     # with offsets to create the final mesh
@@ -281,18 +292,30 @@ def extrude_triangulation(vertices,
                     vertices_3D.copy() + [0.0, 0, height],
                     vertical]
 
-    mesh = Trimesh(*util.append_faces(vertices_seq,
-                                      faces_seq),
-                   process=True,
+    # append sequences into flat nicely indexed arrays
+    vertices, faces = util.append_faces(vertices_seq, faces_seq)
+    if transform is not None:
+        # apply transform here to avoid later bookkeeping
+        vertices = tf.transform_points(
+            vertices, transform)
+        # if the transform flips the winding flip faces back
+        # so that the normals will be facing outwards
+        if tf.flips_winding(transform):
+            # fliplr makes arrays non-contiguous
+            faces = np.ascontiguousarray(np.fliplr(faces))
+    # create mesh object with passed keywords
+    mesh = Trimesh(vertices=vertices,
+                   faces=faces,
                    **kwargs)
-
-    assert mesh.volume > 0.0
+    # only check in strict mode (unit tests)
+    if tol.strict:
+        assert mesh.volume > 0.0
 
     return mesh
 
 
 def triangulate_polygon(polygon,
-                        triangle_args='pq30',
+                        triangle_args=None,
                         engine='auto',
                         **kwargs):
     """
@@ -305,8 +328,8 @@ def triangulate_polygon(polygon,
     ---------
     polygon : Shapely.geometry.Polygon
         Polygon object to be triangulated
-    triangle_args : str
-        Passed to triangle.triangulate
+    triangle_args : str or None
+        Passed to triangle.triangulate i.e: 'p', 'pq30'
     engine : str
         'meshpy', 'triangle', or 'auto'
     kwargs: passed directly to meshpy.triangle.build:
@@ -331,6 +354,10 @@ def triangulate_polygon(polygon,
        Index of vertices that make up triangles
     """
 
+    # set default triangulation if not specified
+    if triangle_args is None:
+        triangle_args = 'p'
+
     # turn the polygon in to vertices, segments, and hole points
     arg = _polygon_to_kwargs(polygon)
 
@@ -440,7 +467,7 @@ def _polygon_to_kwargs(polygon):
         try:
             start += add_boundary(interior, start)
         except BaseException:
-            log.warn('invalid interior, continuing')
+            log.warning('invalid interior, continuing')
             continue
 
     # create clean (n,2) float array of vertices
@@ -481,9 +508,10 @@ def box(extents=None, transform=None, **kwargs):
 
     Returns
     ------------
-    box : trimesh.Trimesh
-      Cuboid geometry in space
+    geometry : trimesh.Trimesh
+      Mesh of a cuboid
     """
+    # vertices of the cube
     vertices = [0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1,
                 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1]
     vertices = np.array(vertices,
@@ -491,6 +519,7 @@ def box(extents=None, transform=None, **kwargs):
                         dtype=np.float64).reshape((-1, 3))
     vertices -= 0.5
 
+    # resize cube based on passed extents
     if extents is not None:
         extents = np.asanyarray(extents, dtype=np.float64)
         if extents.shape != (3,):
@@ -513,6 +542,8 @@ def box(extents=None, transform=None, **kwargs):
                   face_normals=face_normals,
                   process=False,
                   **kwargs)
+
+    # do the transform here to preserve face normals
     if transform is not None:
         box.apply_transform(transform)
 
@@ -544,7 +575,7 @@ def icosahedron():
     return mesh
 
 
-def icosphere(subdivisions=3, radius=1.0):
+def icosphere(subdivisions=3, radius=1.0, color=None):
     """
     Create an isophere centered at the origin.
 
@@ -556,6 +587,8 @@ def icosphere(subdivisions=3, radius=1.0):
       4 ** subdivisions, so you probably want to keep this under ~5
     radius : float
       Desired radius of sphere
+    color: (3,) float or uint8
+      Desired color of sphere
 
     Returns
     ---------
@@ -574,6 +607,8 @@ def icosphere(subdivisions=3, radius=1.0):
         ico = ico.subdivide()
         refine_spherical()
     ico._validate = True
+    if color is not None:
+        ico.visual.face_colors = color
     return ico
 
 
@@ -703,6 +738,7 @@ def capsule(height=1.0,
 def cylinder(radius=1.0,
              height=1.0,
              sections=32,
+             segment=None,
              transform=None,
              **kwargs):
     """
@@ -716,6 +752,10 @@ def cylinder(radius=1.0,
       The height of the cylinder
     sections : int
       How many pie wedges should the cylinder have
+    segment : (2, 3) float
+      Endpoints of axis, overrides transform and height
+    transform : (4, 4) float
+      Transform to apply
     **kwargs:
         passed to Trimesh to create cylinder
 
@@ -725,6 +765,22 @@ def cylinder(radius=1.0,
       Resulting mesh of a cylinder
     """
 
+    if segment is not None:
+        segment = np.asanyarray(segment, dtype=np.float64)
+        if segment.shape != (2, 3):
+            raise ValueError('segment must be 2 3D points!')
+        vector = segment[1] - segment[0]
+        # override height with segment length
+        height = np.linalg.norm(vector)
+        # point in middle of line
+        midpoint = segment[0] + (vector * 0.5)
+        # align Z with our desired direction
+        rotation = align_vectors([0, 0, 1], vector)
+        # translate to midpoint of segment
+        translation = tf.translation_matrix(midpoint)
+        # compound the rotation and translation
+        transform = np.dot(translation, rotation)
+
     # create a 2D pie out of wedges
     theta = np.linspace(0, np.pi * 2, sections)
     vertices = np.column_stack((np.sin(theta),
@@ -902,7 +958,7 @@ def axis(origin_size=0.04,
     axis_origin.visual.face_colors = origin_color
 
     # create the cylinder for the z-axis
-    translation = transformations.translation_matrix(
+    translation = tf.translation_matrix(
         [0, 0, axis_length / 2])
     z_axis = cylinder(
         radius=axis_radius,
@@ -912,10 +968,10 @@ def axis(origin_size=0.04,
     z_axis.visual.face_colors = [0, 0, 255]
 
     # create the cylinder for the y-axis
-    translation = transformations.translation_matrix(
+    translation = tf.translation_matrix(
         [0, 0, axis_length / 2])
-    rotation = transformations.rotation_matrix(np.radians(-90),
-                                               [1, 0, 0])
+    rotation = tf.rotation_matrix(np.radians(-90),
+                                  [1, 0, 0])
     y_axis = cylinder(
         radius=axis_radius,
         height=axis_length,
@@ -924,10 +980,10 @@ def axis(origin_size=0.04,
     y_axis.visual.face_colors = [0, 255, 0]
 
     # create the cylinder for the x-axis
-    translation = transformations.translation_matrix(
+    translation = tf.translation_matrix(
         [0, 0, axis_length / 2])
-    rotation = transformations.rotation_matrix(np.radians(90),
-                                               [0, 1, 0])
+    rotation = tf.rotation_matrix(np.radians(90),
+                                  [0, 1, 0])
     x_axis = cylinder(
         radius=axis_radius,
         height=axis_length,
@@ -943,7 +999,9 @@ def axis(origin_size=0.04,
     return marker
 
 
-def camera_marker(camera, marker_height=0.4, origin_size=None):
+def camera_marker(camera,
+                  marker_height=0.4,
+                  origin_size=None):
     """
     Create a visual marker for a camera object, including an axis and FOV.
 
@@ -964,7 +1022,6 @@ def camera_marker(camera, marker_height=0.4, origin_size=None):
 
     # append the visualizations to an array
     meshes = [axis(origin_size=marker_height / 10.0)]
-    meshes[0].apply_transform(camera.transform)
 
     try:
         # path is a soft dependency
@@ -985,13 +1042,13 @@ def camera_marker(camera, marker_height=0.4, origin_size=None):
     z = marker_height
 
     # combine the points into the vertices of an FOV visualization
-    points = [(0, 0, 0),
-              (-x, -y, z),
-              (x, -y, z),
-              (x, y, z),
-              (-x, y, z)]
-    # apply the camera extrinsic transform
-    points = transformations.transform_points(points, camera.transform)
+    points = np.array(
+        [(0, 0, 0),
+         (-x, -y, z),
+         (x, -y, z),
+         (x, y, z),
+         (-x, y, z)],
+        dtype=float)
 
     # create line segments for the FOV visualization
     # a segment from the origin to each bound of the FOV
