@@ -1,5 +1,4 @@
 import numpy as np
-import networkx as nx
 
 import collections
 import json
@@ -8,6 +7,14 @@ from .. import util
 from .. import visual
 from .. import transformations
 
+try:
+    import networkx as nx
+except BaseException as E:
+    # create a dummy module which will raise the ImportError
+    # or other exception only when someone tries to use networkx
+    from ..exceptions import ExceptionModule
+    nx = ExceptionModule(E)
+
 
 def load_XAML(file_obj, *args, **kwargs):
     """
@@ -154,16 +161,14 @@ def load_3DXML(file_obj, *args, **kwargs):
     Load a 3DXML scene into kwargs.
 
     Parameters
-    -----------
+    ------------
     file_obj : file object
-                  open, containing 3DXML
+      Open and containing 3DXML data
 
     Returns
     -----------
-    geometries : list of dict
-                     kwargs for Trimesh constructor
-    graph      :  list of dict
-                     kwargs for Scene.graph.update
+    kwargs : dict
+      Can be passed to trimesh.exchange.load.load_kwargs
     """
     archive = util.decompress(file_obj, file_type='zip')
 
@@ -432,7 +437,10 @@ def print_element(element):
     ------------
     element : etree element
     """
-    print(etree.tostring(element, pretty_print=True).decode('utf-8'))
+    pretty = etree.tostring(
+        element, pretty_print=True).decode('utf-8')
+    print(pretty)
+    return pretty
 
 
 try:
