@@ -25,11 +25,25 @@ for _name in ['vhacd', 'testVHACD']:
 exists = _vhacd_executable is not None
 
 
-def convex_decomposition(mesh, **kwargs):
+def convex_decomposition(mesh, debug=False, **kwargs):
+    """
+    Run VHACD to generate an approximate convex decomposition
+    of a single mesh.
+
+    Parameters
+    --------------
+    mesh : trimesh.Trimesh
+      Mesh to be decomposed into convex components
+
+    Returns
+    ------------
+    meshes : (n,) trimesh.Trimesh
+      List of convex meshes
+    """
     if not exists:
         raise ValueError('No vhacd available!')
 
-    argstring = ' --input $mesh_0 --output $mesh_post --log $script'
+    argstring = ' --input $MESH_0 --output $MESH_POST --log $SCRIPT'
 
     # pass through extra arguments from the input dictionary
     for key, value in kwargs.items():
@@ -38,6 +52,14 @@ def convex_decomposition(mesh, **kwargs):
 
     with MeshScript(meshes=[mesh],
                     script='',
-                    tmpfile_ext='obj') as vhacd:
+                    tmpfile_ext='obj',
+                    group_material=False,
+                    split_object=True,
+                    debug=debug) as vhacd:
         result = vhacd.run(_vhacd_executable + argstring)
+
+    # if we got a scene back return a list of meshes
+    if hasattr(result, 'geometry') and isinstance(result.geometry, dict):
+        return list(result.geometry.values())
+
     return result
