@@ -1,21 +1,18 @@
 """
 path.py
+-----------
 
-A library designed to work with vector paths.
+A module designed to work with vector paths such as
+those stored in a DXF or SVG file.
 """
-
 import numpy as np
-import networkx as nx
 
 import copy
 import collections
 
-from shapely.geometry import Polygon
-from scipy.spatial import cKDTree as KDTree
-
 from ..points import plane_fit
 from ..geometry import plane_transform
-
+from ..visual import to_rgba
 from ..constants import log
 from ..constants import tol_path as tol
 
@@ -23,19 +20,25 @@ from .util import concatenate
 
 from .. import util
 from .. import units
+from .. import bounds
 from .. import caching
 from .. import grouping
+from .. import exceptions
 from .. import transformations
 
 from . import raster
 from . import repair
 from . import simplify
+from . import creation  # NOQA
 from . import polygons
-from . import segments
+from . import segments  # NOQA
 from . import traversal
 
 from .exchange.export import export_path
 
+from scipy.spatial import cKDTree
+from shapely.geometry import Polygon
+
 try:
     # try running shapely speedups
     # these mostly speed up object instantiation
@@ -43,7 +46,14 @@ try:
     if speedups.available:
         speedups.enable()
 except BaseException:
-    pass
+    log.warning('shapely speedups failed', exc_info=True)
+
+try:
+    import networkx as nx
+except BaseException as E:
+    # create a dummy module which will raise the ImportError
+    # or other exception only when someone tries to use networkx
+    nx = exceptions.ExceptionModule(E)
 
 
 class Path(object):
@@ -57,10 +67,12 @@ class Path(object):
     """
 
     def __init__(self,
-                 entities=[],
-                 vertices=[],
+                 entities=None,
+                 vertices=None,
                  metadata=None,
-                 process=True):
+                 process=True,
+                 colors=None,
+                 **kwargs):
         """
         Instantiate a path object.
 
@@ -78,11 +90,15 @@ class Path(object):
 
         self.entities = entities
         self.vertices = vertices
-        self.metadata = dict()
 
+        # assign each color to each entity
+        self.colors = colors
+        # collect metadata into new dictionary
+        self.metadata = dict()
         if metadata.__class__.__name__ == 'dict':
             self.metadata.update(metadata)
 
+        # cache will dump whenever self.crc changes
         self._cache = caching.Cache(id_function=self.crc)
 
         if process:
@@ -90,46 +106,107 @@ class Path(object):
             # aren't merged properly
             self.merge_vertices()
 
+    def __repr__(self):
+        """
+        Print a quick summary of the number of vertices and entities.
+        """
+        return '<trimesh.{}(vertices.shape={}, len(entities)={})>'.format(
+            type(self).__name__,
+            self.vertices.shape,
+            len(self.entities))
+
     def process(self):
         """
-        Apply basic cleaning functions to the Path object, in- place.
+        Apply basic cleaning functions to the Path object in- place.
         """
-        log.debug('Processing drawing')
         with self._cache:
             for func in self._process_functions():
                 func()
         return self
 
+    @property
+    def colors(self):
+        """
+        Colors are stored per-entity.
+
+        Returns
+        ------------
+        colors : (len(entities), 4) uint8
+          RGBA colors for each entity
+        """
+        # start with default colors
+        colors = np.ones((len(self.entities), 4))
+        colors = (colors * [100, 100, 100, 255]).astype(np.uint8)
+        # collect colors from entities
+        for i, e in enumerate(self.entities):
+            if hasattr(e, 'color') and e.color is not None:
+                colors[i] = to_rgba(e.color)
+        # don't allow parts of the color array to be written
+        colors.flags['WRITEABLE'] = False
+        return colors
+
+    @colors.setter
+    def colors(self, values):
+        """
+        Set the color for every entity in the Path.
+
+        Parameters
+        ------------
+        values : (len(entities), 4) uint8
+          Color of each entity
+        """
+        # if not set return
+        if values is None:
+            return
+        # make sure colors are RGBA
+        colors = to_rgba(values)
+        if len(colors) != len(self.entities):
+            raise ValueError('colors must be per-entity!')
+        # otherwise assign each color to the entity
+        for c, e in zip(colors, self.entities):
+            e.color = c
+
     @property
     def vertices(self):
         return self._vertices
 
     @vertices.setter
     def vertices(self, values):
-        self._vertices = caching.tracked_array(values)
+        self._vertices = caching.tracked_array(
+            values, dtype=np.float64)
 
     @property
     def entities(self):
+        """
+        The actual entities making up the path.
+
+        Returns
+        -----------
+        entities : (n,) trimesh.path.entities.Entity
+          Entities such as Line, Arc, or BSpline curves
+        """
         return self._entities
 
     @entities.setter
     def entities(self, values):
-        self._entities = np.asanyarray(values)
+        if values is None:
+            self._entities = np.array([])
+        else:
+            self._entities = np.asanyarray(values)
 
     @property
     def layers(self):
         """
-        If entities have a layer defined, return it.
+        Get a list of the layer for every entity.
 
         Returns
         ---------
-        layers: (len(entities), ) list of str
+        layers : (len(entities), ) any
+          Whatever is stored in each `entity.layer`
         """
-        layer = ['NONE'] * len(self.entities)
-        for i, e in enumerate(self.entities):
-            if hasattr(e, 'layer'):
-                layer[i] = str(e.layer)
-        return layer
+        # layer is a required property for entities
+        layers = [e.layer for e in self.entities]
+        return layers
 
     def crc(self):
         """
@@ -137,13 +214,14 @@ class Path(object):
 
         Returns
         ------------
-        crc: int, CRC of entity points and vertices
+        crc : int
+          CRC of entity points and vertices
         """
         # first CRC the points in every entity
-        target = caching.crc32(bytes().join(e._bytes()
-                                            for e in self.entities))
-        # add the CRC for the vertices
-        target ^= self.vertices.crc()
+        target = caching.crc32(bytes().join(
+            e._bytes() for e in self.entities))
+        # XOR the CRC for the vertices
+        target ^= self.vertices.fast_hash()
         return target
 
     def md5(self):
@@ -152,12 +230,13 @@ class Path(object):
 
         Returns
         ------------
-        md5: str, two appended MD5 hashes
+        md5 : str
+          Appended MD5 hashes
         """
         # first MD5 the points in every entity
         target = '{}{}'.format(
-            util.md5_object(bytes().join(e._bytes()
-                                         for e in self.entities)),
+            util.md5_object(bytes().join(
+                e._bytes() for e in self.entities)),
             self.vertices.md5())
 
         return target
@@ -169,7 +248,8 @@ class Path(object):
 
         Returns
         ---------
-        paths: (n,) sequence of (*,) int referencing self.entities
+        paths : (n,) sequence of (*,) int
+          Referencing self.entities
         """
         paths = traversal.closed_paths(self.entities,
                                        self.vertices)
@@ -182,7 +262,8 @@ class Path(object):
 
         Returns
         ----------
-        dangling: (n,) int, index of self.entities
+        dangling : (n,) int
+          Index of self.entities
         """
         if len(self.paths) == 0:
             return np.arange(len(self.entities))
@@ -199,10 +280,10 @@ class Path(object):
 
         Returns
         ----------
-        kdtree: scipy.spatial.cKDTree object holding self.vertices
+        kdtree : scipy.spatial.cKDTree
+          Object holding self.vertices
         """
-
-        kdtree = KDTree(self.vertices.view(np.ndarray))
+        kdtree = cKDTree(self.vertices.view(np.ndarray))
         return kdtree
 
     @property
@@ -214,12 +295,25 @@ class Path(object):
         Returns
         ----------
         scale : float
-            Approximate size of the world holding this path
+          Approximate size of the world holding this path
         """
         # use vertices peak-peak rather than exact extents
         scale = float((self.vertices.ptp(axis=0) ** 2).sum() ** .5)
         return scale
 
+    @caching.cache_decorator
+    def length(self):
+        """
+        The total discretized length of every entity.
+
+        Returns
+        --------
+        length: float, summed length of every entity
+        """
+        length = float(sum(i.length(self.vertices)
+                           for i in self.entities))
+        return length
+
     @caching.cache_decorator
     def bounds(self):
         """
@@ -227,7 +321,8 @@ class Path(object):
 
         Returns
         ----------
-        bounds: (2, dimension) float, (min, max) coordinates
+        bounds : (2, dimension) float
+          AABB with (min, max) coordinates
         """
         # get the exact bounds of each entity
         # some entities (aka 3- point Arc) have bounds that can't
@@ -238,11 +333,23 @@ class Path(object):
         # flatten bound extrema into (n, dimension) array
         points = points.reshape((-1, self.vertices.shape[1]))
         # get the max and min of all bounds
-        bounds = np.array([points.min(axis=0),
-                           points.max(axis=0)],
-                          dtype=np.float64)
+        return np.array([points.min(axis=0),
+                         points.max(axis=0)],
+                        dtype=np.float64)
 
-        return bounds
+    @caching.cache_decorator
+    def centroid(self):
+        """
+        Return the centroid of axis aligned bounding box enclosing
+        all entities of the path object.
+
+        Returns
+        -----------
+        centroid : (d,) float
+          Approximate centroid of the path
+        """
+        centroid = self.bounds.mean(axis=0)
+        return centroid
 
     @property
     def extents(self):
@@ -251,7 +358,8 @@ class Path(object):
 
         Returns
         ---------
-        extents: (dimension,) float, edge length of AABB
+        extents : (dimension,) float
+          Edge length of AABB
         """
         return self.bounds.ptp(axis=0)
 
@@ -262,7 +370,8 @@ class Path(object):
 
         Returns
         -----------
-        units: str, current unit system
+        units : str
+          Current unit system
         """
         if 'units' in self.metadata:
             return self.metadata['units']
@@ -279,8 +388,10 @@ class Path(object):
 
         Parameters
         -----------
-        desired: str, unit system to convert to
-        guess:   bool, if True will attempt to guess units
+        desired : str
+          Unit system to convert to
+        guess : bool
+          If True will attempt to guess units
         """
         units._convert_units(self,
                              desired=desired,
@@ -288,12 +399,17 @@ class Path(object):
 
     def explode(self):
         """
-        Turn every multi- segment entity into single segment entities, in- place
+        Turn every multi- segment entity into single segment
+        entities in- place.
         """
-        new_entities = collections.deque()
+        new_entities = []
         for entity in self.entities:
+            # explode into multiple entities
             new_entities.extend(entity.explode())
-        self.entities = np.array(new_entities)
+        # avoid setter and assign new entities
+        self._entities = np.array(new_entities)
+        # explicitly clear cache
+        self._cache.clear()
 
     def fill_gaps(self, distance=0.025):
         """
@@ -314,19 +430,33 @@ class Path(object):
 
         Returns
         -----------
-        closed: every entity is connected at its ends
+        closed : bool
+          Every entity is connected at its ends
         """
         closed = all(i == 2 for i in
                      dict(self.vertex_graph.degree()).values())
 
         return closed
 
+    @property
+    def is_empty(self):
+        """
+        Are any entities defined for the current path.
+
+        Returns
+        ----------
+        empty : bool
+          True if no entities are defined
+        """
+        return len(self.entities) == 0
+
     @caching.cache_decorator
     def vertex_graph(self):
         """
         Return a networkx.Graph object for the entity connectiviy
 
-        graph: networkx.Graph object, holding vertex indexes
+        graph : networkx.Graph
+          Holds vertex indexes
         """
         graph, closed = traversal.vertex_graph(self.entities)
         return graph
@@ -352,8 +482,8 @@ class Path(object):
 
         Parameters
         -----------
-        transform: (d+1, d+1) float, homogenous transformation
-                                    matrix for (n, d) vertices
+        transform : (d+1, d+1) float
+          Homogeneous transformation for vertices
         """
         dimension = self.vertices.shape[1]
         transform = np.asanyarray(transform, dtype=np.float64)
@@ -406,12 +536,12 @@ class Path(object):
 
         Parameters
         -----------
-        transform: (dimension + 1, dimension + 1) float, homogenous
-                   transformation matrix
+        scale : float or (3,) float
+          Scale to be applied to mesh
         """
         dimension = self.vertices.shape[1]
         matrix = np.eye(dimension + 1)
-        matrix[:dimension, :dimension] *= float(scale)
+        matrix[:dimension, :dimension] *= scale
         self.apply_transform(matrix)
 
     def apply_translation(self, offset):
@@ -420,15 +550,15 @@ class Path(object):
 
         Parameters
         -----------
-        transform: (dimension + 1, dimension + 1) float, homogenous
-                   transformation matrix
+        offset : float or (3,) float
+          Translation to be applied to mesh
         """
         # work on 2D and 3D paths
         dimension = self.vertices.shape[1]
         # make sure offset is correct length and type
         offset = np.array(
             offset, dtype=np.float64).reshape(dimension)
-        # create a homogenous transform
+        # create a homogeneous transform
         matrix = np.eye(dimension + 1)
         # apply the offset
         matrix[:dimension, dimension] = offset
@@ -441,7 +571,8 @@ class Path(object):
 
         Parameters
         ------------
-        name: str to apply to each entity
+        name : str
+          Apply layer name to every entity
         """
         for e in self.entities:
             e.layer = name
@@ -453,9 +584,9 @@ class Path(object):
 
         Returns
         -----------
-        matrix: (dimension + 1, dimension + 1) float,
-                    homogenous transformation
-                    that was applied to the current Path object.
+        matrix : (dimension + 1, dimension + 1) float
+          Homogeneous transformation that was applied
+          to the current Path object.
         """
         dimension = self.vertices.shape[1]
         matrix = np.eye(dimension + 1)
@@ -471,14 +602,18 @@ class Path(object):
         --------------
         digits : None, or int
           How many digits to consider when merging vertices
+
         Alters
         -----------
         self.entities : entity.points re- referenced
         self.vertices : duplicates removed
         """
+        if len(self.vertices) == 0:
+            return
         if digits is None:
-            digits = util.decimal_to_digits(tol.merge * self.scale,
-                                            min_digits=1)
+            digits = util.decimal_to_digits(
+                tol.merge * self.scale,
+                min_digits=1)
 
         unique, inverse = grouping.unique_rows(self.vertices,
                                                digits=digits)
@@ -538,12 +673,14 @@ class Path(object):
 
         Parameters
         -----------
-        entity_ids: (n,) int, indexes of self.entities to remove
+        entity_ids : (n,) int
+          Indexes of self.entities to remove
         """
         if len(entity_ids) == 0:
             return
-        kept = np.setdiff1d(np.arange(len(self.entities)), entity_ids)
-        self.entities = np.array(self.entities)[kept]
+        keep = np.ones(len(self.entities))
+        keep[entity_ids] = False
+        self.entities = self.entities[keep]
 
     def remove_invalid(self):
         """
@@ -568,7 +705,7 @@ class Path(object):
         entity_hashes = np.array([hash(i) for i in self.entities])
         unique, inverse = grouping.unique_rows(entity_hashes)
         if len(unique) != len(self.entities):
-            self.entities = np.array(self.entities)[unique]
+            self.entities = self.entities[unique]
 
     @caching.cache_decorator
     def referenced_vertices(self):
@@ -579,7 +716,10 @@ class Path(object):
         -----------
         referenced_vertices: (n,) int, indexes of self.vertices
         """
-        referenced = np.hstack([e.points for e in self.entities])
+        # no entities no reference
+        if len(self.entities) == 0:
+            return np.array([], dtype=np.int64)
+        referenced = np.concatenate([e.points for e in self.entities])
         referenced = np.unique(referenced.astype(np.int64))
 
         return referenced
@@ -669,7 +809,8 @@ class Path(object):
 
         Returns
         ---------
-        copied: Path object, copy of self
+        copied : Path object
+          Copy of self
         """
 
         metadata = {}
@@ -685,7 +826,8 @@ class Path(object):
         # copy the core data
         copied = type(self)(entities=copy.deepcopy(self.entities),
                             vertices=copy.deepcopy(self.vertices),
-                            metadata=metadata)
+                            metadata=metadata,
+                            process=False)
 
         cache = {}
         # try to copy the cache over to the new object
@@ -740,9 +882,7 @@ class Path3D(Path):
     def _process_functions(self):
         return [self.merge_vertices,
                 self.remove_duplicate_entities,
-                self.remove_unreferenced_vertices,
-                self.generate_closed_paths,
-                self.generate_discrete]
+                self.remove_unreferenced_vertices]
 
     def to_planar(self,
                   to_2D=None,
@@ -757,7 +897,7 @@ class Path3D(Path):
         Parameters
         -----------
         to_2D: (4,4) float
-            Homogenous transformation matrix to apply,
+            Homogeneous transformation matrix to apply,
             If not passed a plane will be fitted to vertices.
         normal: (3,) float, or None
            Approximate normal of direction of plane
@@ -777,6 +917,9 @@ class Path3D(Path):
         """
         # which vertices are actually referenced
         referenced = self.referenced_vertices
+        # if nothing is referenced return an empty path
+        if len(referenced) == 0:
+            return Path2D(), np.eye(4)
 
         # no explicit transform passed
         if to_2D is None:
@@ -860,7 +1003,7 @@ class Path3D(Path):
            If False will not execute matplotlib.pyplot.show
         """
         import matplotlib.pyplot as plt
-        from mpl_toolkits.mplot3d import Axes3D
+        from mpl_toolkits.mplot3d import Axes3D  # NOQA
         fig = plt.figure()
         axis = fig.add_subplot(111, projection='3d')
         for discrete in self.discrete:
@@ -879,7 +1022,7 @@ class Path3D(Path):
            If False will not execute matplotlib.pyplot.show
         """
         import matplotlib.pyplot as plt
-        from mpl_toolkits.mplot3d import Axes3D
+        from mpl_toolkits.mplot3d import Axes3D  # NOQA
         fig = plt.figure()
         axis = fig.add_subplot(111, projection='3d')
         for entity in self.entities:
@@ -912,14 +1055,30 @@ class Path2D(Path):
         """
         Transform the current path so that its OBB is axis aligned
         and OBB center is at the origin.
+
+        Returns
+        -----------
+        obb : (3, 3) float
+          Homogeneous transformation matrix
         """
-        if len(self.root) == 1:
-            matrix, bounds = polygons.polygon_obb(
-                self.polygons_closed[self.root[0]])
-            self.apply_transform(matrix)
-            return matrix
-        else:
-            raise ValueError('Not implemented for multibody geometry')
+        matrix = self.obb
+        self.apply_transform(matrix)
+        return matrix
+
+    @caching.cache_decorator
+    def obb(self):
+        """
+        Get a transform that centers and aligns the OBB of the
+        referenced vertices with the XY axis.
+
+        Returns
+        -----------
+        obb : (3, 3) float
+          Homogeneous transformation matrix
+        """
+        matrix = bounds.oriented_bounds_2D(
+            self.vertices[self.referenced_vertices])[0]
+        return matrix
 
     def rasterize(self,
                   pitch,
@@ -1089,18 +1248,6 @@ class Path2D(Path):
         area = float(sum(i.area for i in self.polygons_full))
         return area
 
-    @caching.cache_decorator
-    def length(self):
-        """
-        The total discretized length of every entity.
-
-        Returns
-        --------
-        length: float, summed length of every entity
-        """
-        length = float(sum(i.length(self.vertices) for i in self.entities))
-        return length
-
     def extrude(self, height, **kwargs):
         """
         Extrude the current 2D path into a 3D mesh.
@@ -1183,10 +1330,20 @@ class Path2D(Path):
         if resolution is None:
             resolution = self.scale / 1000.0
 
-        medials = [polygons.medial_axis(i, resolution, clip)
-                   for i in self.polygons_full]
-        medials = np.sum(medials)
-        return medials
+        # convert the edges to Path2D kwargs
+        from .exchange.misc import edges_to_path
+
+        # edges and vertices
+        edge_vert = [polygons.medial_axis(i, resolution, clip)
+                     for i in self.polygons_full]
+        # create a Path2D object for each region
+        medials = [Path2D(**edges_to_path(
+            edges=e, vertices=v)) for e, v in edge_vert]
+
+        # get a single Path2D of medial axis
+        medial = concatenate(medials)
+
+        return medial
 
     def connected_paths(self, path_id, include_self=False):
         """
@@ -1226,24 +1383,25 @@ class Path2D(Path):
         """
         return simplify.simplify_basic(self, **kwargs)
 
-    def simplify_spline(self, path_indexes=None, smooth=.0002):
+    def simplify_spline(self, smooth=.0002, verbose=False):
         """
         Convert paths into b-splines.
 
         Parameters
         -----------
-        path_indexes : (n) int
-          List of indexes of self.paths to convert
         smooth : float
           How much the spline should smooth the curve
+        verbose : bool
+          Print detailed log messages
 
         Returns
         ------------
-        simplified: Path2D object
+        simplified : Path2D
+          Discrete curves replaced with splines
         """
         return simplify.simplify_spline(self,
-                                        path_indexes=path_indexes,
-                                        smooth=smooth)
+                                        smooth=smooth,
+                                        verbose=verbose)
 
     def split(self):
         """
@@ -1290,6 +1448,7 @@ class Path2D(Path):
                    'Arc0': {'color': 'r', 'linewidth': 1},
                    'Arc1': {'color': 'b', 'linewidth': 1},
                    'Bezier0': {'color': 'k', 'linewidth': 1},
+                   'Bezier1': {'color': 'k', 'linewidth': 1},
                    'BSpline0': {'color': 'm', 'linewidth': 1},
                    'BSpline1': {'color': 'm', 'linewidth': 1}}
         for entity in self.entities:
@@ -1353,7 +1512,7 @@ class Path2D(Path):
         ---------
         root: (n,) int, list of indexes
         """
-        populate = self.enclosure_directed
+        populate = self.enclosure_directed  # NOQA
         return self._cache['root']
 
     @caching.cache_decorator
