@@ -1,8 +1,6 @@
 import numpy as np
-import networkx as nx
 
-from shapely.geometry import Polygon, Point
-from shapely import vectorized
+from shapely.geometry import Polygon
 
 from rtree import Rtree
 from collections import deque
@@ -17,6 +15,14 @@ from ..transformations import transform_points
 
 from .traversal import resample_path
 
+try:
+    import networkx as nx
+except BaseException as E:
+    # create a dummy module which will raise the ImportError
+    # or other exception only when someone tries to use networkx
+    from ..exceptions import ExceptionModule
+    nx = ExceptionModule(E)
+
 
 def enclosure_tree(polygons):
     """
@@ -46,7 +52,7 @@ def enclosure_tree(polygons):
     for i, polygon in enumerate(polygons):
         # if a polygon is None it means creation
         # failed due to weird geometry so ignore it
-        if polygon is None:
+        if polygon is None or len(polygon.bounds) != 4:
             continue
         # insert polygon bounds into rtree
         tree.insert(i, polygon.bounds)
@@ -54,10 +60,8 @@ def enclosure_tree(polygons):
         contains.add_node(i)
 
     # loop through every polygon
-    for i, polygon in enumerate(polygons):
-        # if polygon creation failed ignore it
-        if polygon is None:
-            continue
+    for i in contains.nodes():
+        polygon = polygons[i]
         # we first query for bounding box intersections from the R-tree
         for j in tree.intersection(polygon.bounds):
             # if we are checking a polygon against itself continue
@@ -105,12 +109,15 @@ def edges_to_polygons(edges, vertices):
 
     Parameters
     -----------
-    edges: (n,2) int, indexes of vertices which represent lines
-    vertices: (m,2) float, vertex positions
+    edges : (n, 2) int
+      Indexes of vertices which represent lines
+    vertices : (m, 2) float
+      Vertices in 2D space
 
     Returns
     ----------
-    polygons: (p,) list of shapely.geometry.Polygon objects
+    polygons : (p,) shapely.geometry.Polygon
+      Polygon objects with interiors
     """
 
     # create closed polygon objects
@@ -160,14 +167,17 @@ def polygon_obb(polygon):
 
     Parameters
     -------------
-    polygons: shapely.geometry.Polygon
+    polygons : shapely.geometry.Polygon
+      Input geometry
 
     Returns
     -------------
-    transform: (3,3) float, transformation matrix
-               which will move input polygon from its original position
-               to the first quadrant where the AABB is the OBB
-    extents:   (2,) float, extents of transformed polygon
+    transform : (3, 3) float
+      Transformation matrix
+      which will move input polygon from its original position
+      to the first quadrant where the AABB is the OBB
+    extents : (2,) float
+      Extents of transformed polygon
     """
     if hasattr(polygon, 'exterior'):
         points = np.asanyarray(polygon.exterior.coords)
@@ -180,20 +190,19 @@ def polygon_obb(polygon):
 
 def transform_polygon(polygon, matrix):
     """
-    Transform a polygon by a a 2D homogenous transform.
+    Transform a polygon by a a 2D homogeneous transform.
 
     Parameters
     -------------
     polygon : shapely.geometry.Polygon
-                 2D polygon to be transformed.
+      2D polygon to be transformed.
     matrix  : (3, 3) float
-                 2D homogenous transformation.
+      2D homogeneous transformation.
 
     Returns
     --------------
     result : shapely.geometry.Polygon
-                 Polygon transformed by matrix.
-
+      Polygon transformed by matrix.
     """
     matrix = np.asanyarray(matrix, dtype=np.float64)
 
@@ -250,14 +259,18 @@ def resample_boundaries(polygon, resolution, clip=None):
 
     Parameters
     -------------
-    polygon:    shapely.geometry.Polygon object
-    resolution: float, desired distance between points on boundary
-    clip:       (2,) int, upper and lower bounds to clip
-                number of samples to (to avoid exploding counts)
+    polygon : shapely.geometry.Polygon
+      Source geometry
+    resolution : float
+      Desired distance between points on boundary
+    clip : (2,) int
+      Upper and lower bounds to clip
+      number of samples to avoid exploding count
 
     Returns
     ------------
-    kwargs: dict, keyword args for a Polygon(**kwargs)
+    kwargs : dict
+     Keyword args for a Polygon constructor `Polygon(**kwargs)`
     """
     def resample_boundary(boundary):
         # add a polygon.exterior or polygon.interior to
@@ -283,11 +296,13 @@ def stack_boundaries(boundaries):
 
     Parameters
     ------------
-    boundaries: dict, with keys 'shell', 'holes'
+    boundaries : dict
+      With keys 'shell', 'holes'
 
     Returns
     ------------
-    stacked: (n, 2) float, list of vertices
+    stacked : (n, 2) float
+      Stacked vertices
     """
     if len(boundaries['holes']) == 0:
         return boundaries['shell']
@@ -307,27 +322,26 @@ def medial_axis(polygon,
     Parameters
     ----------
     polygon : shapely.geometry.Polygon
-        The source geometry
+      The source geometry
     resolution : float
-        Distance between each sample on the polygon boundary
-    clip : None, or (2,) float
-        Clip the lower and upper bound of sample count to:
-        [minimum number of samples, maximum number of samples]
-        specifying a very fine resolution can cause the sample count to
-        explode, so clip specifies a minimum and maximum number of samples
-        to use per boundary region. To not clip, this can be specified as:
-        [0, np.inf]
+      Distance between each sample on the polygon boundary
+    clip : None, or (2,) int
+      Clip sample count to min of clip[0] and max of clip[1]
 
     Returns
     ----------
-    medial : Path2D object
+    edges : (n, 2) int
+      Vertex indices representing line segments
+      on the polygon's medial axis
+    vertices : (m, 2) float
+      Vertex positions in space
     """
     from scipy.spatial import Voronoi
-    from .path import Path2D
-    from .exchange.misc import edges_to_path
+    from shapely import vectorized
 
     if resolution is None:
-        resolution = .01
+        resolution = np.reshape(
+            polygon.bounds, (2, 2)).ptp(axis=0).max() / 100
 
     # get evenly spaced points on the polygons boundaries
     samples = resample_boundaries(polygon=polygon,
@@ -345,71 +359,8 @@ def medial_axis(polygon,
     ridge = np.asanyarray(voronoi.ridge_vertices, dtype=np.int64)
     # only take ridges where every vertex is contained
     edges = ridge[contains[ridge].all(axis=1)]
-    # line objects from edges
-    medial = Path2D(**edges_to_path(
-        edges=edges,
-        vertices=voronoi.vertices))
 
-    return medial
-
-
-class InversePolygon:
-    """
-    Create an inverse polygon.
-
-    The primary use case is that given a point inside a polygon,
-    you want to find the minimum distance to the boundary of the polygon.
-    """
-
-    def __init__(self, polygon):
-        _DIST_BUFFER = .05
-
-        # create a box around the polygon
-        bounds = (np.array(polygon.bounds))
-        bounds += (_DIST_BUFFER * np.array([-1, -1, 1, 1]))
-        coord_ext = bounds[
-            np.array([2, 1, 2, 3, 0, 3, 0, 1, 2, 1])].reshape((-1, 2))
-        # set the interior of the box to the exterior of the polygon
-        coord_int = [np.array(polygon.exterior.coords)]
-
-        # a box with an exterior- shaped hole in it
-        exterior = Polygon(shell=coord_ext,
-                           holes=coord_int)
-        # make exterior polygons out of all of the interiors
-        interiors = [Polygon(i.coords) for i in polygon.interiors]
-
-        # save these polygons to a flat list
-        self._polygons = np.append(exterior, interiors)
-
-    def distances(self, point):
-        """
-        Find the minimum distances from a point to the exterior and interiors
-
-        Parameters
-        ---------
-        point: (2) list or shapely.geometry.Point
-
-        Returns
-        ---------
-        distances: (n) list of floats
-        """
-        distances = [i.distance(Point(point)) for i in self._polygons]
-        return distances
-
-    def distance(self, point):
-        """
-        Find the minimum distance from a point to the boundary of the polygon.
-
-        Parameters
-        ---------
-        point: (2) list or shapely.geometry.Point
-
-        Returns
-        ---------
-        distance: float
-        """
-        distance = np.min(self.distances(point))
-        return distance
+    return edges, voronoi.vertices
 
 
 def polygon_hash(polygon):
@@ -444,12 +395,15 @@ def random_polygon(segments=8, radius=1.0):
 
     Parameters
     ---------
-    segments: int, the maximum number of sides the random polygon will have
-    radius:   float, the approximate radius of the polygon desired
+    segments : int
+      The maximum number of sides the random polygon will have
+    radius : float
+      The approximate radius of the polygon desired
 
     Returns
     ---------
-    polygon: shapely.geometry.Polygon object with random exterior, and no interiors.
+    polygon : shapely.geometry.Polygon
+      Geometry object with random exterior and no interiors.
     """
     angles = np.sort(np.cumsum(np.random.random(
         segments) * np.pi * 2) % (np.pi * 2))
@@ -465,15 +419,17 @@ def random_polygon(segments=8, radius=1.0):
 
 def polygon_scale(polygon):
     """
-    For a Polygon object, return the diagonal length of the AABB.
+    For a Polygon object return the diagonal length of the AABB.
 
     Parameters
     ------------
-    polygon: shapely.geometry.Polygon object
+    polygon : shapely.geometry.Polygon
+      Source geometry
 
     Returns
     ------------
-    scale: float, length of AABB diagonal
+    scale : float
+      Length of AABB diagonal
     """
     extents = np.reshape(polygon.bounds, (2, 2)).ptp(axis=0)
     scale = (extents ** 2).sum() ** .5
@@ -489,15 +445,15 @@ def paths_to_polygons(paths, scale=None):
     Parameters
     -----------
     paths : (n,) sequence
-        Of (m,2) float, closed paths
+      Of (m, 2) float closed paths
     scale: float
-        Approximate scale of drawing for precision
+      Approximate scale of drawing for precision
 
     Returns
     -----------
-    polys: (p,) list
-        shapely.geometry.Polygon
-        None
+    polys : (p,) list
+      Filled with Polygon or None
+
     """
     polygons = [None] * len(paths)
     for i, path in enumerate(paths):
@@ -525,23 +481,25 @@ def sample(polygon, count, factor=1.5, max_iter=10):
     Parameters
     -----------
     polygon : shapely.geometry.Polygon
-                Polygon that will contain points
-    count   : int
-                Number of points to return
-    factor  : float
-                How many points to test per loop
-                IE, count * factor
-    max_iter : int,
-                Maximum number of intersection loops
-                to run, total points sampled is
-                count * factor * max_iter
+      Polygon that will contain points
+    count : int
+      Number of points to return
+    factor : float
+      How many points to test per loop
+    max_iter : int
+      Maximum number of intersection checks is:
+      > count * factor * max_iter
 
     Returns
     -----------
     hit : (n, 2) float
-           Random points inside polygon
-           where n <= count
+      Random points inside polygon
+      where n <= count
     """
+    # do batch point-in-polygon queries
+    from shapely import vectorized
+
+    # get size of bounding box
     bounds = np.reshape(polygon.bounds, (2, 2))
     extents = bounds.ptp(axis=0)
 
@@ -578,17 +536,22 @@ def repair_invalid(polygon, scale=None, rtol=.5):
 
     Parameters
     -----------
-    polygon: shapely.geometry.Polygon object
-    rtol:    float, how close does a perimeter have to be
-    scale:   float, or None
+    polygon : shapely.geometry.Polygon
+      Source geometry
+    rtol : float
+      How close does a perimeter have to be
+    scale : float or None
+      For numerical precision reference
 
     Returns
     ----------
-    repaired: shapely.geometry.Polygon object
+    repaired : shapely.geometry.Polygon
+      Repaired polygon
 
     Raises
     ----------
-    ValueError: if polygon can't be repaired
+    ValueError
+      If polygon can't be repaired
     """
     if hasattr(polygon, 'is_valid') and polygon.is_valid:
         return polygon
@@ -607,9 +570,9 @@ def repair_invalid(polygon, scale=None, rtol=.5):
         return basic
 
     if scale is None:
-        distance = tol.buffer * polygon_scale(polygon)
+        distance = 0.002 * polygon_scale(polygon)
     else:
-        distance = tol.buffer * scale
+        distance = 0.002 * scale
 
     # if there are no interiors, we can work with just the exterior
     # ring, which is often more reliable
@@ -626,6 +589,21 @@ def repair_invalid(polygon, scale=None, rtol=.5):
                                              rtol=rtol):
                 return recon
 
+        # try de-deuplicating the outside ring
+        points = np.array(polygon.exterior)
+        # remove any segments shorter than tol.merge
+        # this is a little risky as if it was discretized more
+        # finely than 1-e8 it may remove detail
+        unique = np.append(True, (np.diff(points, axis=0)**2).sum(
+            axis=1)**.5 > 1e-8)
+        # make a new polygon with result
+        dedupe = Polygon(shell=points[unique])
+        # check result
+        if dedupe.is_valid and np.isclose(dedupe.length,
+                                          polygon.length,
+                                          rtol=rtol):
+            return dedupe
+
     # buffer and unbuffer the whole polygon
     buffered = polygon.buffer(distance).buffer(-distance)
     # if it returned multiple polygons check the largest
